  --
  -- Developer : Andrew Tunnecliffe 
  -- Contact: andrew.207@gmail.com http://www.atunnecliffe.com
  -- Date : Jan-2014
  -- All Code (C) 2014 Andrew Tunnecliffe
  --

function love.load()
    debug = false;
    gravityFlag = true;
    g = love.graphics
    playerColor = {255,0,128}
    groundColor = {25,200,25}
    CLIMBING_ABILITY = 5
    dt = 0
    
    -- instantiate our player and set initial values
    require "player"
    require "platform"
    require "camera"
    p = player:new()
    cam = camera:new(p, -WIDTH/4, -WIDTH+WIDTH/4)    
    
    p.x = 300
    p.y = 300
    p.width = 25
    p.height = 40
    p.jumpSpeed = -800
    p.runSpeed = 500
    
    gravity = 1800
    
    ground = platform:new(0, 500, 8000, 100);
    block = platform:new(200, 400, 100, 20);
    block2 = platform:new(400, 300, 100, 20);
    blocks = {}
    blocks[1] = ground
    blocks[2] = block
    blocks[3] = block2
    blocks[4] = platform:new(700, 350, 30, 150)
    blocks[5] = platform:new(550, 320, 30, 150)
    blocks[6] = platform:new(900, 300, 30, 150)
    blocks[7] = platform:new(1100, 200, 30, 200)
    blocks[8] = platform:new(1101, 400, 200, 20)
    
    for _,v in ipairs(blocks) do 
      v.current["above"] = 0
      v.current["below"] = 0
      v.current["left"] = 0
      v.current["right"] = 0
      v.last["above"] = 0
      v.last["below"] = 0
      v.last["left"] = 0
      v.last["right"] = 0
    end
end
 
function love.update(dt)
    if love.keyboard.isDown("right") then
        p:moveRight()
        cam:movingRight()
    end
    if love.keyboard.isDown("left") then
        p:moveLeft()
        cam:movingLeft()
    end
  if debug then
    if not love.keyboard.isDown("z") then
     if love.keyboard.isDown("d") then
         p.x = p.x + 1
      end
      if love.keyboard.isDown("a") then
          p.x = p.x - 1
      end
      if love.keyboard.isDown("w") then
          p.y = p.y - 1
      end
      if love.keyboard.isDown("s") then
          p.y = p.y + 1
      end
    end
  end
  
  dt = dt
  
    -- if the x key is pressed...
    if love.keyboard.isDown("x") then
    -- make the player jump
        p:jump()
    end
 
    -- update the player's position
    p:update(dt, gravity)
    cam:update(dt)
 
    -- stop the player when they hit the borders
    p.collided = false;
    for _,v in ipairs(blocks) do 
      p:compareLocation(v, dt)
    end
end
 
function love.draw()
  love.graphics.translate(cam.x, cam.y)
  
  -- draw the player shape
  g.setColor(playerColor)
  p:draw();
  
  -- draw the ground
  g.setColor(groundColor)
  for _,v in ipairs(blocks) do 
    v:draw()
  end
  
  -- debug information
  if (debug) then
    g.setColor(255, 255, 255)
    local isTrue = ""
    g.print("Player coordinates: ("..math.floor(p.x).." - "..math.floor(p.x+p.width)..", "..math.floor(p.y).." - "..math.floor(p.y+p.height)..")", -cam.x, -cam.y + 5)
    g.print("Current state: "..p.state, -cam.x, -cam.y + 20)
    g.print("Speeds: "..p.xSpeed..", "..p.ySpeed, -cam.x, -cam.y + 35)
  end  
end
 
function love.keyreleased(key)
  if key == "escape" then
    love.event.push("kill")   
  end
  if (key == "right") or (key == "left") then
    p.decelarating = true;
  end
  
  if (key == "`") then
    debug = not debug
  end
  if (key == "tab") then
    gravityFlag = not gravityFlag
  end
   if (key == "q") then
    p.x = 10 p.y = 10
  end
  
  -- debug movement
  if love.keyboard.isDown("z") then
    if key == "d" then
      p.x = p.x + 1
    end
    if key == "a" then
      p.x = p.x - 1
    end
    if key == "w" then
      p.y = p.y - 1
    end
    if key == "s" then
      p.y = p.y + 1
    end
  end    
end